package com.icomsys.util;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class CommonBeanUtils {

    public static <T> T convertType(Object source, Class<T> targetType, String... ignoreProperties) {
        try {
            // T 타입의 객체 생성
            T target = targetType.getDeclaredConstructor().newInstance();

            if (source != null) {
                List<String> ignoredProperties = List.of(ignoreProperties);
                Field[] sourceFields = source.getClass().getDeclaredFields();
                Field[] targetFields = targetType.getDeclaredFields();

                for (Field sourceField : sourceFields) {
                    sourceField.setAccessible(true);
                    if (ignoredProperties.contains(sourceField.getName())) {
                        continue;
                    }

                    for (Field targetField : targetFields) {
                        if (sourceField.getName().equals(targetField.getName()) &&
                                sourceField.getType().equals(targetField.getType())) {
                            targetField.setAccessible(true);
                            targetField.set(target, sourceField.get(source));
                        }
                    }
                }
            }
            return target;
        } catch (Exception ex) {
            throw new RuntimeException("객체 변환 실패", ex);
        }
    }

    public static <T> List<T> convertListType(List<?> sourceList, Class<T> targetType, String... ignoreProperties) {
        List<T> targetList = new ArrayList<>();
        if (sourceList != null) {
            for (Object source : sourceList) {
                T target = convertType(source, targetType, ignoreProperties);
                targetList.add(target);
            }
        }
        return targetList;
    }

    @SuppressWarnings("unchecked")
    public static <T> T getProperty(Object obj, String propertyName) {
        try {
            Field field = obj.getClass().getDeclaredField(propertyName);
            field.setAccessible(true);
            return (T) field.get(obj);
        } catch (NoSuchFieldException | IllegalAccessException | ClassCastException e) {
            throw new RuntimeException("속성 읽기 실패", e);
        }
    }

    public static List<String> getPropertyNames(Object obj) {
        return getPropertyNames(obj, null);
    }

    public static List<String> getPropertyNames(Object obj, Class<?> findType) {
        Field[] fields = obj.getClass().getDeclaredFields();
        Stream<Field> fieldStream = Stream.of(fields);

        if (findType != null) {
            fieldStream = fieldStream.filter(field -> field.getType().equals(findType));
        }

        return fieldStream.map(Field::getName).collect(Collectors.toList());
    }
}
